"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServiceFactory = exports.DEFAULT_PRIORITY = void 0;
const extend_1 = require("../util/extend");
exports.DEFAULT_PRIORITY = {
    L1: 'High',
    L2: 'Medium',
    L3: 'Low',
};
/**
 * 多客户端工厂实现
 */
class ServiceFactory {
    constructor() {
        this.clients = new Map();
        this.options = {};
    }
    async initClients(options = {}) {
        this.options = options;
        // merge options.client to options.clients['default']
        if (options.client) {
            options.clients = options.clients || {};
            options.clients['default'] = options.clients['default'] || {};
            (0, extend_1.extend)(true, options.clients['default'], options.client);
        }
        // multi client
        if (options.clients) {
            for (const id of Object.keys(options.clients)) {
                await this.createInstance(options.clients[id], id);
            }
        }
        // set priority
        this.priority = options.priority || {
            ...Array.from(this.clients.keys()).map(name => ({
                [name]: exports.DEFAULT_PRIORITY.L2,
            })),
        };
    }
    get(id = 'default') {
        return this.clients.get(id);
    }
    has(id) {
        return this.clients.has(id);
    }
    async createInstance(config, clientName) {
        // options.default will be merge in to options.clients[id]
        config = (0, extend_1.extend)(true, {}, this.options['default'], config);
        const client = await this.createClient(config, clientName);
        if (client) {
            if (clientName) {
                this.clients.set(clientName, client);
            }
            return client;
        }
    }
    async destroyClient(client) { }
    async stop() {
        for (const value of this.clients.values()) {
            await this.destroyClient(value);
        }
    }
    getDefaultClientName() {
        return this.options['defaultClientName'];
    }
    getClients() {
        return this.clients;
    }
    getClientKeys() {
        return Array.from(this.clients.keys());
    }
    getClientPriority(clientName) {
        return this.priority[clientName] || exports.DEFAULT_PRIORITY.L2;
    }
    isHighPriority(clientName) {
        return this.getClientPriority(clientName) === exports.DEFAULT_PRIORITY.L1;
    }
    isMediumPriority(clientName) {
        return this.getClientPriority(clientName) === exports.DEFAULT_PRIORITY.L2;
    }
    isLowPriority(clientName) {
        return this.getClientPriority(clientName) === exports.DEFAULT_PRIORITY.L3;
    }
}
exports.ServiceFactory = ServiceFactory;
//# sourceMappingURL=serviceFactory.js.map